﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using System.Reflection;
using System.IO;



namespace InformacjeOZestawie
{
    public static class Zestaw
    {
        
        public static void SporzadzRaport(Assembly asm, StreamWriter sw)
        {
            sw.WriteLine("***Informacje o zestawie {0}***\n", asm.FullName);
            sw.WriteLine("Powyższy zestaw zawiera następujące typy:");
            Type[] typy = asm.GetTypes();
            foreach (Type typ in typy)
            {
                InformacjeOTypie(typ, sw);
            }
        }

        public static void InformacjeOTypie(Type typ, StreamWriter sw)
        {
            sw.WriteLine(" - {0}",typ.Name);
            sw.WriteLine("   zdefiniowany jest w przestrzeni nazw: {0}", typ.Namespace);
            sw.WriteLine("   typ bazowy: {0}", typ.BaseType.FullName);
            sw.WriteLine("   czy typ jest typem publicznm: {0}", typ.IsPublic);
            sw.WriteLine("   czy typ jest typem zagnieżdżonym: {0}", typ.IsNested);
            sw.WriteLine("   czy typ jest typem ogólnym: {0}", typ.IsGenericType);
            if (typ.IsClass)
            {
                sw.Write("   typ jest klasą ");
                if(typ.IsAbstract)
                    sw.Write("   abstrakcyjną ");
                if (typ.IsSealed)
                    sw.Write("   zamkniętą ");
                sw.WriteLine();
            }
            if (typ.IsInterface)
            {
                sw.WriteLine("   typ jest interfejsem");
            }
            if (typ.IsValueType)
            {
                sw.WriteLine("   typ jest typem bezpośrednim");
            }
            
            ZaimplementowaneInterfejsy(typ, sw);
            ListaKonstruktorow(typ, sw);
            ZmienneSkladowe(typ, sw);
            MetodySkladowe(typ, sw);
            Zdarzenia(typ, sw);
            sw.WriteLine("-------------------------------------------------------");
        }

        public static void ZaimplementowaneInterfejsy(Type typ, StreamWriter sw)
        {
            Type[] interfejsy = typ.GetInterfaces();
            if (interfejsy != null && interfejsy.Length > 0)
            {
                sw.WriteLine("   typ implementuje następujące interfejsy:");
                foreach (Type t in interfejsy)
                {
                    sw.WriteLine("     - {0}",t.FullName);
                }
            }
        }

        public static void ListaKonstruktorow(Type typ, StreamWriter sw)
        {
            ConstructorInfo[] konstr = typ.GetConstructors(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Static | BindingFlags.Instance);
            if (konstr != null && konstr.Length > 0)
            {
                sw.WriteLine("   typ zwiera następujące konstruktory:");
                foreach (ConstructorInfo k in konstr)
                {
                    if (k.IsPrivate)
                        sw.Write("     - private");
                    if (k.IsPublic)
                        sw.Write("     - public");
                    if (k.IsFamily)
                        sw.Write("     - protected");
                    if (k.IsAssembly)
                        sw.Write("     - internal");
                    if (k.IsFamilyOrAssembly)
                        sw.Write("     - internal protected");
                    if (k.IsStatic)
                        sw.Write(" static ");
                    
                    sw.WriteLine(" {0} ", k.Name);
                    ParametryMetody(k, sw);
                }
            }
        }

        public static void ZmienneSkladowe(Type typ, StreamWriter sw)
        {
            FieldInfo[] pola = typ.GetFields(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Static | BindingFlags.Instance );
            if (pola != null && pola.Length>0)
            {
                sw.WriteLine("   typ zwiera następujące pola:");
                foreach (FieldInfo fi in pola)
                {
                    if(fi.IsPrivate)
                        sw.Write("     - private");
                    if (fi.IsPublic)
                        sw.Write("     - public");
                    if(fi.IsFamily)
                        sw.Write("     - protected");
                    if (fi.IsAssembly)
                        sw.Write("     - internal");
                    if (fi.IsFamilyOrAssembly)
                        sw.Write("     - internal protected");
                    if (fi.IsLiteral)
                        sw.Write(" const, wartość: ", fi.GetValue(null));
                    if (fi.IsInitOnly)
                        sw.Write(" readonly ");
                    if (fi.IsStatic)
                        sw.Write(" static ");
                    
                    sw.WriteLine(" {0} {1};", fi.FieldType.FullName, fi.Name);                    
                }
            }
        }
        public static void MetodySkladowe(Type typ, StreamWriter sw)
        {
            MethodInfo[] pola = typ.GetMethods(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Static | BindingFlags.Instance);
            if (pola != null && pola.Length > 0)
            {
                sw.WriteLine("   typ zwiera następujące metody:");
                foreach (MethodInfo mi in pola)
                {
                    if (mi.IsPrivate)
                        sw.Write("     - private");
                    if (mi.IsPublic)
                        sw.Write("     - public");
                    if (mi.IsFamily)
                        sw.Write("     - protected");
                    if (mi.IsAssembly)
                        sw.Write("     - internal");
                    if (mi.IsFamilyOrAssembly)
                        sw.Write("     - internal protected");
                    if (mi.IsStatic)
                        sw.Write(" static ");
                    if (mi.IsAbstract)
                        sw.Write(" abstract ");
                    if (mi.IsFinal)
                        sw.Write(" sealed ");
                    if (mi.IsVirtual)
                        sw.Write(" virtual ");           
                    sw.WriteLine(" {0} typ zwracany: {1}", mi.Name, mi.ReturnType);
                    ParametryMetody(mi, sw);
                }
            }
        }
        public static void ParametryMetody(MethodBase mi, StreamWriter sw)
        {
            ParameterInfo[] parametry = mi.GetParameters();
            if (parametry != null && parametry.Length > 0)
            {
                sw.WriteLine("       Do metody przesyłane są następujące parametry:");
                foreach (ParameterInfo p in parametry)
                {
                    sw.WriteLine("       - typ: {0}, nazwa: {1}",p.ParameterType.FullName, p.Name);                    
                }
            }
        }

        public static void Zdarzenia(Type typ, StreamWriter sw)
        {
            EventInfo[] zdarzenia = typ.GetEvents(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Static | BindingFlags.Instance);
            if (zdarzenia != null && zdarzenia.Length > 0)
            {
                sw.WriteLine("   typ zawiera następujące zdarzenia:");
                foreach (EventInfo z in zdarzenia)
                {
                    sw.WriteLine("     - {0} {1}",z.EventHandlerType.FullName, z.Name );
                }
            }
        }
    }
}
